@@ -160,85 +160,125 @@ c              ++++++++++++++
           Move.Track.pos.y = Move.Track.pos.y + dispmr.y
           Move.Track.pos.z = Move.Track.pos.z + dispmr.z
        endif
-c               direction
-       Move.Track.w.x = Move.Track.w.x + dispmd.x
-       Move.Track.w.y = Move.Track.w.y + dispmd.y
-       Move.Track.w.z = Move.Track.w.z + dispmd.z
-c           normalize
-       norm = sqrt(Move.Track.w.x**2 +
-     *            Move.Track.w.y**2 +
-     *            Move.Track.w.z**2)
+       Move.Track.w.x = dispmd.x
+       Move.Track.w.y = dispmd.y
+       Move.Track.w.z = dispmd.z
 
-       Move.Track.w.x = Move.Track.w.x/norm
-       Move.Track.w.y = Move.Track.w.y/norm
-       Move.Track.w.z = Move.Track.w.z/norm
 
       end
       subroutine epdefByE
       end
 
+c      Compute magnetic deflection (angle and displacement)
 c      
 c     ************************************************  
-      subroutine epmagneticDef(aTrack, B, len, dr,  dd)
+      subroutine epmagneticDef(aTrack, B, leng, dr,  dir)
 c     ************************************************  
-c      Compute magnetic deflection (angle and displacement)
-c   Note that:
-c      dr must be added  as  newpos = oldpos + len * olddir + dr
-c      dd may be added  as  newdir = olddir + dd ; then normalize newdi.
-c  This is a small modification of Cosmos function.
-c     In Cosmos,  length is m, B is T
- 
       implicit none
 #include "ZepTrack.h"
 #include "Zep3Vec.h"
- 
+
       record /epTrack/ aTrack     ! input. a charged ptcl
       record /ep3Vec/ B            ! input. magnetic field vector. in T
                                   !  in local coordinate
-      real*8  len         ! input. length  travelled in cm
-      record /epPos/  dr  ! output. displacement vector in cm
-      record /epDirec/dd  ! output. direction cos displacement.
-c                 Note. This is approx that  beta * dbeta =0
-c                 (orthogonal).  dd is dbeta/beta. and should be
-c                 small enough.
-      
+      real*8  leng         ! input. length  travelled in cm
+      record /ep3Vec/  dr  ! output. displacement vector in cm
+      record /epDirec/dir  ! output.  new direction cos 
+c    This is exact if B is const.
+c   Note that:
+c      dr must be added  as  newpos = oldpos +  dr
+c      dir is the new direction cos.  These two are differenct
+c      from the old version.
+c
+
+c
+      record /epPos/ Tx, Ty, Tz, w, r
+
+      real*8 Babs, sint, cost, rgt, sinl, cosl
       real*8   pabs, temp
+c
+c       get abs(B)
+      Babs = sqrt( B.x**2 + B.y**2 + B.z**2 )
+      if(Babs .gt. 0.) then
+c            p in GeV
+         pabs = sqrt(aTrack.p.fm.p(4)**2 - aTrack.p.mass**2)
+         if(pabs .gt. 0.) then
+c              transverse gyroradius in cm. B in T, p in GeV
+c              For negative charge, rgt < 0. This is o.k
+            rgt = 3.3358d2*pabs/aTrack.p.charge/Babs
+
+            Tz.x = B.x/Babs
+            Tz.y = B.y/Babs
+            Tz.z = B.z/Babs
+
+            call cscalerProd(aTrack.w, B, cost)
+            cost =max( min(cost/Babs, 1.0d0), -1.d0)
+            sint = sqrt(1.0d0-cost**2)
+            if(abs(sint) .gt. 1.d-6) then            
+               call cvecProd(aTrack.w, Tz, Tx)
+               Tx.x = Tx.x/sint
+               Tx.y = Tx.y/sint
+               Tx.z = Tx.z/sint
+               call cvecProd(Tz, Tx, Ty)
+c////////////////
+c               call checkuv('Tx', sint, Tx)
+c               call checkuv('Ty', sint, Ty)
+c               call checkuv('Tz', sint, Tz)
+c               call checkorth('TxTy', sint, Tx, Ty)
+c               call checkorth('TxTz', sint, Tx, Tz)
+c//////////////////////               
+            else
+               Tx.x = 1.d0
+               Tx.y = 0.
+               Tx.z = 0.
+               Ty.x = 0.
+               Ty.y = 1.d0
+               Ty.z = 0.
+            endif
+c            get wz0=wz
+            call cscalerProd(Tz, aTrack.w, w.z)
+            temp = leng /rgt
+            sinl = sin(temp)         ! this may be < 0
+            cosl = cos(temp)
+
+            w.x = sint*sinl
+            w.y = sint*cosl
+c///////////
+c            call checkuv('w ', sint, w)
+c///////////
+            r.x = - rgt*sint*(cosl - 1.d0)
 
 
-      record /ep3Vec/ dxB    !  direction x B
-      
-      dxB.x = aTrack.w.y * B.z - 
-     *    aTrack.w.z * B.y
-      dxB.y = aTrack.w.z * B.x - 
-     *    aTrack.w.x * B.z
-      dxB.z = aTrack.w.x * B.y -
-     *     aTrack.w.y * B.x
+            r.y = rgt*sint*sinl
+            r.z = leng*w.z
+            
+c                convert to original system.
+            dr.x =
+     *          Tx.x*r.x + Ty.x*r.y + Tz.x*r.z
+            dr.y =
+     *          Tx.y*r.x + Ty.y*r.y + Tz.y*r.z
+            dr.z =
+     *          Tx.z*r.x + Ty.z*r.y + Tz.z*r.z
 
-c      write(*, *) ' v x B', dxB.x, dxB.y, dxB.z
 
-      call cpxyzp(aTrack.p.fm, pabs)   ! |p|
-      if(pabs .eq. 0.) then
-         temp = 0.
+            dir.x =
+     *          Tx.x*w.x + Ty.x*w.y + Tz.x*w.z
+            dir.y =
+     *          Tx.y*w.x + Ty.y*w.y + Tz.y*w.z
+            dir.z =
+     *          Tx.z*w.x + Ty.z*w.y + Tz.z*w.z
+         else
+            dr.x = 0.
+            dr.y = 0.
+            dr.z = 0.
+            dir = aTrack.w
+         endif
       else
-c         temp = len**2 *0.3/2  * aTrack.p.charge /pabs ;  cosmos
-c             ( len*10^-2) * 100.  makes len*10^-2
-         temp = len**2 *0.3d-2 /2  * aTrack.p.charge /pabs  ! cm->m
-
+         dr.x = leng*aTrack.w.x
+         dr.y = leng*aTrack.w.y
+         dr.z = leng*aTrack.w.z
+         dir = aTrack.w
       endif
-      dr.x = dxB.x * temp
-      dr.y = dxB.y * temp
-      dr.z = dxB.z * temp
-
-      if(pabs .gt. 0.) then
-c         temp = 0.3 *len * aTrack.p.charge /pabs  ;   cosmos
-          temp = 0.3d-2 *len * aTrack.p.charge /pabs  
-      else
-         temp = 0.
-      endif
-      dd.x = dxB.x * temp
-      dd.y = dxB.y * temp
-      dd.z = dxB.z * temp
-
       end
 
 
