@@ -79,7 +79,7 @@ c      *********************
 c               init for nuclin, hadrin
 cc       call haddenC
 cc       call chanwnC
-       call cintModels('epics')
+         call cintModels('epics')  
 
 c       ################
 c                user dependent init
@@ -563,7 +563,8 @@ c          set cosmos condition(TrackBefMove, TargetMassN)
       tokgpm2 = Media(MediaNo).X0g *10.d0 ! r.l --> kg/m^2
 c                 target mass and Z; not used from v7.0
 c         but projectile is set. 
-      call ep2cosCond(Media(MediaNo).Aeff, Media(MediaNo).Zeff)
+c      call ep2cosCond(Media(MediaNo).Aeff, Media(MediaNo).Zeff)
+      call ep2cosCond
       call cfixModel( cTrack.p )
       call ciniSmpIntL
 c        call csampNEPIntL  before v7.0
@@ -663,7 +664,7 @@ c                      heavy ion and ke/n is < 0.7 GeV
             endif
 
             if(Media(MediaNo).BirksC1 .gt. 0.) then
-               if( abs(cTrack.p.charge) .eq. 1) then
+               if( abs(cTrack.p.charge) .gt. 1) then
 c                   quenching effect
                   call epOrgCorrec(Media(MediaNo), cTrack.p, dedx, cf)
                else
@@ -858,7 +859,8 @@ c            call epphotoEp(Media(MediaNo).pe,  E, prob, tphot)
 c          set cosmos condition(TrackBefMove, TargetMassN)
 c                 target mass and Z; thi is not used but
 c              projectile is set
-         call ep2cosCond(Media(MediaNo).Aeff, Media(MediaNo).Zeff)
+c         call ep2cosCond(Media(MediaNo).Aeff, Media(MediaNo).Zeff)
+         call ep2cosCond
          call cfixModel( cTrack.p )
          call cghPath(E,  tgp)  ! cosmos function
          tgp = tgp * 0.1   !         to g/cm2
