@@ -85,7 +85,7 @@ c            inquire form
         endif
 
 
-c        call epqcnf(orgw, abcw)
+        call epqcnf(orgw, abcw)
 
 c        call epqworld(nwld, wstruct) 
 c
@@ -124,7 +124,7 @@ c         ---------------- Error -----------------
        implicit none
 #include  "ZepManager.h"
        character(*) file
-       character*12  inpcond
+       character*30  inpcond
        integer klena,icon
 
        open(Ioprim, file=file(1:klena(file) ))
@@ -162,6 +162,7 @@ c         ---------------- Error -----------------
        if(icon .ne. 0) then
           call cerrorMsg("Primary file has no #----- line", 0)
        endif
+
        end
        subroutine ssetip2( aTrack, icon)
        implicit none
@@ -171,7 +172,7 @@ c         ---------------- Error -----------------
        record /epTrack/ aTrack
        integer icon, nf, fc
        integer code, subcode, chg 
-       real*8 erg, xin, yin, zin, wx, wy, wz, u
+       real*8 erg, xin, yin, zin, wx, wy, wz, u, norm
        character*80 buf
        character*20 field(11)
 
@@ -210,6 +211,11 @@ c         ---------------- Error -----------------
           read( field(fc), *)   wy
           fc = fc + 1
           read( field(fc), *)   wz
+c              the acuracy of input data may not be enough
+          norm = sqrt( wx**2 + wy**2 + wz**2 )
+ 1        wx = wx/norm
+          wy = wy/norm
+          wz = wz/norm
        endif
        if( Inptime .gt. 0 ) then
           fc = fc + 1
@@ -348,6 +354,7 @@ c      *****************
       implicit none
 #include  "ZepManager.h"
 #include  "ZsepManager.h"
+#include  "ZepTrackp.h"
 #include  "ZepTrack.h"
 #include  "Zep3Vec.h"
 #include  "Zswk.h"
@@ -356,7 +363,12 @@ c      *****************
       record /epTrack/ aTrack
       record /epTrack/InciTrack, bTrack
       integer ns, i, j, icon
+      logical ok
+      integer ntry
       real*8 sume
+      record /epPos/ org, rxyz
+
+
 
 c             set incident ptcl
 c               fix energy (momentum yet)
@@ -369,13 +381,54 @@ c               fix energy (momentum yet)
       endif
 
 c               fix position
-      if(Inpxyz .eq. 0) then
-         call sfixp(aTrack)
-      endif
+      ok = .false.
+
+      ntry =0
+
+      do while (.not. ok)
+         if(Inpxyz .eq. 0) then
+            call sfixp(aTrack)
+         endif
 c               fix angle
-      if(Inpdir .eq. 0) then
-         call sfixa(aTrack)
-      endif
+         if(Inpdir .eq. 0) then
+            call sfixa(aTrack, ok)
+            if(ok .and. InputP(1:3) .eq. 'sph') then
+c                  pos is not yet fixed.              
+               call epqwcoord(org, rxyz)
+               call epuSphere(rxyz.x, aTrack.w,
+     *           aTrack.pos)
+               aTrack.pos.x= aTrack.pos.x + org.x
+               aTrack.pos.y= aTrack.pos.y + org.y
+               aTrack.pos.z= aTrack.pos.z + org.z
+            endif
+         elseif(InputP(2:4) .eq. 'sph' .or.
+     *     InputP(1:3) .eq. 'sph' ) then
+            call epqwcoord(org, rxyz)
+            if(InputP(1:3) .eq. 'sph' ) then
+               call epuSphere(rxyz.x, aTrack.w,
+     *           aTrack.pos)
+               aTrack.pos.x= aTrack.pos.x + org.x
+               aTrack.pos.y= aTrack.pos.y +  org.y
+               aTrack.pos.z= aTrack.pos.z +  org.z
+               ok = .true.
+            else
+               ok =aTrack.w.x*(aTrack.pos.x -org.x)+
+     *          aTrack.w.y*(aTrack.pos.y -org.y)+
+     *          aTrack.w.z*(aTrack.pos.z -org.z) .lt. 0.
+            endif
+c/////////////
+c            if(.not. ok) then
+c               ntry = ntry +1
+c               if(ntry .gt. 10000) then
+c                  write(0,*) ' ntry=',ntry, ' wz=',aTrack.w.z
+c                  stop 9999
+c               endif
+c            endif
+c//////////////////
+         else
+            ok =.true.
+         endif
+      enddo
 c               fix others
 
       call sfixo(aTrack)
@@ -458,7 +511,7 @@ c             get  current random no.
        real*8 u, r,   dx, dy, cs, sn
        real*8 st, pw
        record /epPos/ org, rxyz
-
+       logical ok
        character*70 msg
 
 
@@ -518,6 +571,9 @@ c
           aTrack.pos.x = aTrack.pos.x + org.x
           aTrack.pos.y = aTrack.pos.y + org.y
           aTrack.pos.z = aTrack.pos.z + org.z
+       elseif(InputP(1:3) .eq. 'sph') then
+c               this case you have to InpDir!=0
+c            wait until angle is given                   
        elseif(InputP(1:1) .eq. 'g') then
 c                  gaussian density beam
           call epGaussb(Hwhm, ProfR, dx, dy)
@@ -544,8 +600,9 @@ c                 around Xinp, Yinp
        endif
        return
 c       ****************************************
-        entry sfixa(aTrack)
+        entry sfixa(aTrack, ok)
 c          
+        ok = .true.
         if(InputA .eq. 'fix') then
            aTrack.w.x = DCInp.x
            aTrack.w.y = DCInp.y
@@ -565,7 +622,7 @@ c              normalize so
            elseif(InputA(1:3) .eq. 'cos') then
               read(InputA(4:12), *) pw
               call rndc(u)
-              if(pw .ne. 1.d0) then
+              if(pw .ne. -1.d0) then
                  DCInp.z=
      *          ( (IMAG_P(CosNormal)**(pw+1.d0) - 
      *                  real(CosNormal)**(pw+1.d0))* u
@@ -595,10 +652,11 @@ c              normalize so
                  aTrack.w.x = DCInp.x
                  aTrack.w.y = DCInp.y
                  aTrack.w.z = DCInp.z
-              else
-                 aTrack.w.x = -DCInp.x
-                 aTrack.w.y = -DCInp.y
-                 aTrack.w.z = -DCInp.z
+               else
+                 ok = .false.
+c                 aTrack.w.x = -DCInp.x
+c                 aTrack.w.y = -DCInp.y
+c                 aTrack.w.z = -DCInp.z
               endif                 
 
 c              direct the angle around the normal vector
